#!/usr/bin/env python

from os.path import dirname, abspath
import sys

import django
from django.conf import settings


if not settings.configured:
    settings.configure(
        INSTALLED_APPS=(
            'localeurl',
            'localeurl.tests',
            'django.contrib.sites', # for sitemap test
            'django.contrib.sessions', # for USE_SESSION
        ),
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3"
            }
        },
        ROOT_URLCONF='localeurl.tests.test_urls',
        SITE_ID=1,
    )


if django.VERSION >= (1, 7):
    django.setup()


def runtests(*test_args):
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)
    try:
        from django.test.runner import DiscoverRunner

        def run_tests(test_args, verbosity, interactive):
            runner = DiscoverRunner(
                verbosity=verbosity, interactive=interactive, failfast=False)
            return runner.run_tests(test_args)
    except ImportError:
        if not test_args:
            test_args = ['tests']
        try:
            from django.test.simple import DjangoTestSuiteRunner

            def run_tests(test_args, verbosity, interactive):
                runner = DjangoTestSuiteRunner(
                    verbosity=verbosity, interactive=interactive, failfast=False)
                return runner.run_tests(test_args)
        except ImportError:
            from django.test.simple import run_tests
    failures = run_tests(
        test_args, verbosity=1, interactive=True)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



"""
Test utilities.

"""
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from django.conf import settings as django_settings
from django.core.handlers.wsgi import WSGIRequest
from django import template
from django.test import Client
from django.utils import encoding


NO_SETTING = object()

class TestSettingsManager(object):
    """
    A class which can modify some Django settings temporarily for a
    test and then revert them to their original values later.

    Based on the work by 'carljm':
    http://www.djangosnippets.org/snippets/1011/
    """
    def __init__(self, settings=django_settings):
        self._settings = settings
        self._original_settings = {}


    def set(self, **kwargs):
        self.set_from_dict(kwargs)


    def set_from_dict(self, settings):
        for k,v in settings.iteritems():
            self._original_settings.setdefault(k,
                    getattr(self._settings, k, NO_SETTING))
            setattr(self._settings, k, v)


    def revert(self):
        for k,v in self._original_settings.iteritems():
            if v == NO_SETTING:
                try:
                    delattr(self._settings, k)
                except AttributeError:
                    # Django < r11825
                    delattr(self._settings._wrapped, k)
            else:
                setattr(self._settings, k, v)
        self._original_settings = {}



class FakePayload(object):
    """
    A wrapper around StringIO that restricts what can be read since data from
    the network can't be seeked and cannot be read outside of its content
    length. This makes sure that views can't do anything under the test client
    that wouldn't work in Real Life.
    """
    def __init__(self, content):
        self.__content = StringIO(content)
        self.__len = len(content)

    def read(self, num_bytes=None):
        if num_bytes is None:
            num_bytes = self.__len or 0
        assert self.__len >= num_bytes, "Cannot read more than the available bytes from the HTTP incoming data."
        content = self.__content.read(num_bytes)
        self.__len -= num_bytes
        return content


class RequestFactory(Client):
    """
    Class that lets you create mock Request objects for use in testing.

    Usage:

    rf = RequestFactory()
    get_request = rf.get('/hello/')
    post_request = rf.post('/submit/', {'foo': 'bar'})

    This class re-uses the django.test.client.Client interface, docs here:
    http://www.djangoproject.com/documentation/testing/#the-test-client

    Once you have a request object you can pass it to any view function,
    just as if that view had been hooked up using a URLconf.

    Based on the work by Simon Willison:
    http://www.djangosnippets.org/snippets/963/
    """
    def request(self, **request):
        """
        Similar to parent class, but returns the request object as soon as it
        has created it.
        """
        environ = {
            'HTTP_COOKIE': self.cookies,
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'wsgi.version':      (1,0),
            'wsgi.url_scheme':   'http',
            'wsgi.input':        FakePayload(''),
            'wsgi.errors':       self.errors,
            'wsgi.multiprocess': True,
            'wsgi.multithread':  False,
            'wsgi.run_once':     False,
        }
        environ.update(self.defaults)
        environ.update(request)
        return WSGIRequest(environ)


class TestTemplate(template.Template):
    """
    TestTemplate behaves just like django.template.Template, but you can give
    it a list of template.Libraries to load before parsing the template. This
    is equivalent to adding a bunch of {% load %} tags to the beginning of your
    template string, but you can use custom tag libraries which do not belong
    to Django applications' templatetags packages.

    Based on the work by Alexander Khodyrev:
    http://www.djangosnippets.org/snippets/1641/
    """
    def __init__(self, template_string, name='<Unknown Template>',
            libraries=[]):
        try:
            template_string = encoding.smart_unicode(template_string)
        except UnicodeDecodeError:
            raise template.TemplateEncodingError(
                    "template content must be unicode or UTF-8 string")
        origin = template.StringOrigin(template_string)
        self.nodelist = self.my_compile_string(template_string, origin,
                libraries)
        self.name = name

    def my_compile_string(self, template_string, origin, libraries=[]):
        "Compiles template_string into NodeList ready for rendering"
        lexer = template.Lexer(template_string, origin)
        parser = template.Parser(lexer.tokenize())
        for lib in libraries:
            parser.add_library(lib)
        return parser.parse()



from django.conf import settings
from django.core import urlresolvers
from django.utils import translation
from localeurl import utils

def reverse(*args, **kwargs):
    reverse_kwargs = kwargs.get('kwargs') or {}
    locale = utils.supported_language(reverse_kwargs.pop(
            'locale', translation.get_language()))
    url = django_reverse(*args, **kwargs)
    _, path = utils.strip_script_prefix(url)
    return utils.locale_url(path, locale)

django_reverse = None

def patch_reverse():
    """
    Monkey-patches the urlresolvers.reverse function. Will not patch twice.
    """
    global django_reverse
    if urlresolvers.reverse is not reverse:
        django_reverse = urlresolvers.reverse
        urlresolvers.reverse = reverse

if settings.USE_I18N:
    patch_reverse()



from django.conf import settings
import django.core.exceptions
from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect
from django.utils import translation
from django.utils.encoding import iri_to_uri
from django.utils.translation.trans_real import parse_accept_lang_header
from localeurl import settings as localeurl_settings
# Importing models ensures that reverse() is patched soon enough. Refs #5.
from localeurl import models, utils

# Make sure the default language is in the list of supported languages
assert utils.supported_language(settings.LANGUAGE_CODE) is not None, \
        "Please ensure that settings.LANGUAGE_CODE is in settings.LANGUAGES."

class LocaleURLMiddleware(object):
    """
    Middleware that sets the language based on the request path prefix and
    strips that prefix from the path. It will also automatically redirect any
    path without a prefix, unless PREFIX_DEFAULT_LOCALE is set to True.
    Exceptions are paths beginning with MEDIA_URL and/or STATIC_URL (if
    settings.LOCALE_INDEPENDENT_MEDIA_URL and/or
    settings.LOCALE_INDEPENDENT_STATIC_URL are set) or matching any regular
    expression from LOCALE_INDEPENDENT_PATHS from the project settings.

    For example, the path '/en/admin/' will set request.LANGUAGE_CODE to 'en'
    and request.path to '/admin/'.

    Alternatively, the language is set by the first component of the domain
    name. For example, a request on 'fr.example.com' would set the language to
    French.

    If you use this middleware the django.core.urlresolvers.reverse function
    is be patched to return paths with locale prefix (see models.py).
    """
    def __init__(self):
        if not settings.USE_I18N:
            raise django.core.exceptions.MiddlewareNotUsed()

    def process_request(self, request):
        locale, path = utils.strip_path(request.path_info)
        if localeurl_settings.USE_SESSION and not locale:
            slocale = request.session.get('django_language')
            if slocale and utils.supported_language(slocale):
                locale = slocale
        if localeurl_settings.USE_ACCEPT_LANGUAGE and not locale:
            accept_lang_header = request.META.get('HTTP_ACCEPT_LANGUAGE', '')
            header_langs = parse_accept_lang_header(accept_lang_header)
            accept_langs = [
                l for l in
                (utils.supported_language(lang[0]) for lang in header_langs)
                if l
            ]
            if accept_langs:
                locale = accept_langs[0]
        locale_path = utils.locale_path(path, locale)
        # locale case might be different in the two paths, that doesn't require
        # a redirect (besides locale they'll be identical anyway)
        if locale_path.lower() != request.path_info.lower():
            locale_url = utils.add_script_prefix(locale_path)

            qs = request.META.get("QUERY_STRING", "")
            if qs:
                # Force this to remain a byte-string by encoding locale_path
                # first to avoid Unicode tainting - downstream will need to
                # handle the job of handling in-the-wild character encodings:
                locale_url = "%s?%s" % (locale_path.encode("utf-8"), qs)

            redirect_class = HttpResponsePermanentRedirect
            if not localeurl_settings.LOCALE_REDIRECT_PERMANENT:
                redirect_class = HttpResponseRedirect
            # @@@ iri_to_uri for Django 1.0; 1.1+ do it in HttpResp...Redirect
            return redirect_class(iri_to_uri(locale_url))
        request.path_info = path
        if not locale:
            try:
                locale = request.LANGUAGE_CODE
            except AttributeError:
                locale = settings.LANGUAGE_CODE
        translation.activate(locale)
        request.LANGUAGE_CODE = translation.get_language()

    def process_response(self, request, response):
        if 'Content-Language' not in response:
            response['Content-Language'] = translation.get_language()
        translation.deactivate()
        return response



from django import template
from django.template import Node, Token, TemplateSyntaxError
from django.template import resolve_variable
from django.template.defaultfilters import stringfilter
from django.templatetags import future

from localeurl import utils

register = template.Library()



def chlocale(url, locale):
    """
    Changes the URL's locale prefix if the path is not locale-independent.
    Otherwise removes locale prefix.
    """
    _, path = utils.strip_script_prefix(url)
    _, path = utils.strip_path(path)
    return utils.locale_url(path, locale)


chlocale = stringfilter(chlocale)
register.filter('chlocale', chlocale)



def rmlocale(url):
    """Removes the locale prefix from the URL."""
    script_prefix, path = utils.strip_script_prefix(url)
    _, path = utils.strip_path(path)
    return ''.join([script_prefix, path])


rmlocale = stringfilter(rmlocale)
register.filter('rmlocale', rmlocale)



def locale_url(parser, token):
    """
    Renders the url for the view with another locale prefix. The syntax is
    like the 'url' tag, only with a locale before the view.

    Examples:
      {% locale_url "de" cal.views.day day %}
      {% locale_url "nl" cal.views.home %}
      {% locale_url "en-gb" cal.views.month month as month_url %}
    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise TemplateSyntaxError("'%s' takes at least two arguments:"
                " the locale and a view" % bits[0])
    urltoken = Token(token.token_type, bits[0] + ' ' + ' '.join(bits[2:]))
    urlnode = future.url(parser, urltoken)
    return LocaleURLNode(bits[1], urlnode)


class LocaleURLNode(Node):
    def __init__(self, locale, urlnode):
        self.locale = locale
        self.urlnode = urlnode

    def render(self, context):
        locale = resolve_variable(self.locale, context)
        if utils.supported_language(locale) is None:
            raise ValueError("locale not in settings.LANGUAGES: %s" % locale)
        path = self.urlnode.render(context)
        if self.urlnode.asvar:
            self.urlnode.render(context)
            context[self.urlnode.asvar] = chlocale(context[self.urlnode.asvar],
                    locale)
            return ''
        else:
            return chlocale(path, locale)


register.tag('locale_url', locale_url)



try:
    from urlparse import urlsplit
except ImportError:
    from urllib.parse import urlsplit
from django import http
from django.utils.translation import check_for_language
from localeurl import utils
from localeurl import settings as localeurl_settings

def change_locale(request):
    """
    Redirect to a given url while changing the locale in the path
    The url and the locale code need to be specified in the
    request parameters.
    """
    next = request.REQUEST.get('next', None)
    if not next:
        referrer = request.META.get('HTTP_REFERER', None)
        if referrer:
            next = urlsplit(referrer)[2]
    if not next:
        next = '/'
    _, path = utils.strip_path(next)
    if request.method == 'POST':
        locale = request.POST.get('locale', None)
        if locale and check_for_language(locale):
            if localeurl_settings.USE_SESSION:
                request.session['django_language'] = locale
            path = utils.locale_path(path, locale)

    response = http.HttpResponseRedirect(path)
    return response



import re
from django.conf import settings

SUPPORTED_LOCALES = dict(
    (code.lower(), name) for code, name in settings.LANGUAGES)
# Issue #15. Sort locale codes to avoid matching e.g. 'pt' before 'pt-br'
LOCALES_RE = '|'.join(
    sorted(SUPPORTED_LOCALES.keys(), key=lambda i: len(i), reverse=True))
PATH_RE = re.compile(r'^/(?P<locale>%s)(?P<path>.*)$' % LOCALES_RE, re.I)

LOCALE_INDEPENDENT_PATHS = [re.compile(p) for p in
                            getattr(settings, 'LOCALE_INDEPENDENT_PATHS', [])]

LOCALE_INDEPENDENT_MEDIA_URL = getattr(settings,
        'LOCALE_INDEPENDENT_MEDIA_URL', True)

LOCALE_INDEPENDENT_STATIC_URL = getattr(settings,
        'LOCALE_INDEPENDENT_STATIC_URL', True)

PREFIX_DEFAULT_LOCALE = getattr(settings, 'PREFIX_DEFAULT_LOCALE', True)

USE_ACCEPT_LANGUAGE = getattr(settings, 'LOCALEURL_USE_ACCEPT_LANGUAGE', False)

USE_SESSION = getattr(settings, 'LOCALEURL_USE_SESSION', False)

LOCALE_REDIRECT_PERMANENT = getattr(settings, 'LOCALE_REDIRECT_PERMANENT', True)



from django.conf import settings
from django.core import urlresolvers
from localeurl import settings as localeurl_settings


def is_locale_independent(path):
    """
    Returns whether the path is locale-independent.
    """
    if (localeurl_settings.LOCALE_INDEPENDENT_MEDIA_URL and
            settings.MEDIA_URL and
            path.startswith(settings.MEDIA_URL)):
        return True
    if (localeurl_settings.LOCALE_INDEPENDENT_STATIC_URL and
            getattr(settings, "STATIC_URL", None) and
            path.startswith(settings.STATIC_URL)):
        return True
    for regex in localeurl_settings.LOCALE_INDEPENDENT_PATHS:
        if regex.search(path):
            return True
    return False


def strip_path(path):
    """
    Separates the locale prefix from the rest of the path. If the path does not
    begin with a locale it is returned without change.
    """
    check = localeurl_settings.PATH_RE.match(path)
    if check:
        path_info = check.group('path') or '/'
        if path_info.startswith('/'):
            return check.group('locale'), path_info
    return '', path


def supported_language(locale):
    """
    Returns the supported language (from settings.LANGUAGES) for the locale.
    """
    locale = locale.lower()
    bits = locale.split('-')
    if locale in localeurl_settings.SUPPORTED_LOCALES:
        return locale
    elif bits[0] in localeurl_settings.SUPPORTED_LOCALES:
        return bits[0]
    else:
        return None


def is_default_locale(locale):
    """
    Returns whether the locale is the default locale.
    """
    return locale == supported_language(settings.LANGUAGE_CODE)


def locale_path(path, locale=''):
    """
    Generate the localeurl-enabled path from a path without locale prefix. If
    the locale is empty settings.LANGUAGE_CODE is used.
    """
    locale = supported_language(locale)
    if not locale:
        locale = supported_language(settings.LANGUAGE_CODE)
    if is_locale_independent(path):
        return path
    elif is_default_locale(locale) and not localeurl_settings.PREFIX_DEFAULT_LOCALE:
        return path
    else:
        return ''.join([u'/', locale, path])


def locale_url(path, locale=''):
    """
    Generate the localeurl-enabled URL from a path without locale prefix. If
    the locale is empty settings.LANGUAGE_CODE is used.
    """
    path = locale_path(path, locale)
    return add_script_prefix(path)


def strip_script_prefix(url):
    """
    Strips the SCRIPT_PREFIX from the URL. Because this function is meant for
    use in templates, it assumes the URL starts with the prefix.
    """
    assert url.startswith(urlresolvers.get_script_prefix()), \
            "URL must start with SCRIPT_PREFIX: %s" % url
    pos = len(urlresolvers.get_script_prefix()) - 1
    return url[:pos], url[pos:]


def add_script_prefix(path):
    """
    Prepends the SCRIPT_PREFIX to a path.
    """

    return ''.join([urlresolvers.get_script_prefix(), path[1:]])



import warnings

warnings.warn(
    "The localeurl_future templatetag library is deprecated; "
    "use localeurl_tags instead.",
    DeprecationWarning
    )

from django import template

from localeurl_tags import locale_url


register = template.Library()


register.tag('locale_url', locale_url)



"""
URLconf for testing.
"""
try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('localeurl.tests.test_urls',
     url(r'^dummy/$', 'dummy', name='dummy0'),
     url(r'^dummy/(?P<test>.+)$', 'dummy', name='dummy1'),
)

def dummy(request, test='test'):
    pass



from django.contrib.sitemaps import Sitemap

from localeurl.templatetags.localeurl_tags import chlocale

class LocaleurlSitemap(Sitemap):
    """
    From a `snippet by tomas`_.

    .. _`snippet by tomas`: http://www.djangosnippets.org/snippets/1620/
    """
    def __init__(self, language):
        self.language = language
        
    def location(self, obj):
        return chlocale(obj.get_absolute_url(), self.language)



try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url
from localeurl.views import change_locale

urlpatterns = patterns('',
	url(r'^change/', change_locale, name='localeurl_change_locale'),
)



#!/usr/bin/env python
#
# Copyright (c) 2008 Joost Cassee
# Licensed under the terms of the MIT License (see LICENSE.txt)

from setuptools import setup
import metadata

app_name = metadata.name
version = metadata.version

long_description = open('docs/index.rst').read().split('split here', 1)[0] + """

See the `full documentation`_.

.. _`full documentation`: http://django-localeurl.readthedocs.org
"""

setup(
    name = "django-%s" % app_name,
    version = version,

    packages = [app_name, '%s.templatetags' % app_name, '%s.tests' % app_name],

    author = "Joost Cassee",
    author_email = "joost@cassee.net",
    maintainer = "Carl Meyer",
    maintainer_email = "carl@oddbird.net",
    description = "A Django application that allow you to specify the" \
            " language of a page in the URL.",
    long_description = long_description,
    license = "MIT License",
    keywords = "django i18n",
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
    ],
    url = "http://django-%s.readthedocs.org/" % app_name,
    test_suite = 'runtests.runtests',
)



# -*- coding: utf-8 -*-
"""
Tests for the localeurl application.
"""
import re

from django import template
from django.core import urlresolvers
from django.test import TestCase
from django.utils import translation

from localeurl import middleware, settings as localeurl_settings, utils
from localeurl.models import reverse
from localeurl.sitemaps import LocaleurlSitemap
from localeurl.templatetags import localeurl_tags
from localeurl.tests import test_utils


def settings_fixture(mgr):
    mgr.set(
        USE_I18N=True,
        LANGUAGES=(
            ('en', 'English'),
            ('nl-nl', 'Dutch'),
            ('nl-be', 'Flemish'),
            ('fr', 'French'),
            ('pt', 'Portuguese'),
            ('pt-br', 'Brazilian Portuguese'),
            ('fi', 'Finnish'),
            ('fil', 'Filipino'),
        ),
        LANGUAGE_CODE='en-gb',
        LOCALE_INDEPENDENT_PATHS=(
            re.compile('^/$'),
            '^/test/independent/',
        ),
        LOCALE_INDEPENDENT_MEDIA_URL=True,
        MEDIA_URL='/media/',
        LOCALE_INDEPENDENT_STATIC_URL=True,
        STATIC_URL='/static/',
        TEMPLATE_CONTEXT_PROCESSORS=(
            'django.core.context_processors.i18n',
        ),
    )


class LocaleurlTestCase(TestCase):
    urls = 'localeurl.tests.test_urls'

    def setUp(self):
        self.settings_manager = test_utils.TestSettingsManager()
        settings_fixture(self.settings_manager)
        translation.activate("en")
        reload(localeurl_settings)

    def tearDown(self):
        self.settings_manager.revert()
        reload(localeurl_settings)


class UtilsTestCase(LocaleurlTestCase):
    def test_is_locale_independent(self):
        self.assertFalse(utils.is_locale_independent('/fr/about'))
        self.assertFalse(utils.is_locale_independent('/about'))
        self.assertTrue(utils.is_locale_independent('/media/img/logo.png'))
        self.assertTrue(utils.is_locale_independent('/static/img/logo.png'))
        self.assertTrue(utils.is_locale_independent('/'))
        self.assertTrue(utils.is_locale_independent(
            '/test/independent/bla/bla'))

    def test_strip_path(self):
        self.assertEqual(('', '/'), utils.strip_path('/'))
        self.assertEqual(('', '/about/'), utils.strip_path('/about/'))
        self.assertEqual(('', '/about/localeurl/'),
                         utils.strip_path('/about/localeurl/'))
        self.assertEqual(('fr', '/about/localeurl/'),
                         utils.strip_path('/fr/about/localeurl/'))
        self.assertEqual(('nl-be', '/about/localeurl/'),
                         utils.strip_path('/nl-be/about/localeurl/'))
        self.assertEqual(('', '/de/about/localeurl/'),
                         utils.strip_path('/de/about/localeurl/'))

    def test_strip_path_takes_longer_code_first(self):
        # Refs issue #15.
        self.assertEqual(('pt-br', '/about/localeurl/'),
                         utils.strip_path('/pt-br/about/localeurl/'))

    def test_supported_language(self):
        self.assertEqual('fr', utils.supported_language('fr'))
        self.assertEqual('nl-be', utils.supported_language('nl-be'))
        self.assertEqual('en', utils.supported_language('en-gb'))
        self.assertEqual('fil', utils.supported_language('fil-foo'))
        self.assertEqual(None, utils.supported_language('de'))

    def test_is_default_locale(self):
        self.assertTrue(utils.is_default_locale('en'))
        self.assertFalse(utils.is_default_locale('en-gb'))
        self.assertFalse(utils.is_default_locale('fr'))
        self.assertFalse(utils.is_default_locale('de'))

    def test_locale_path(self):
        self.assertEqual('/en/about/localeurl/',
                utils.locale_path('/about/localeurl/'))
        self.assertEqual('/en/about/localeurl/',
                utils.locale_path('/about/localeurl/', 'de'))
        self.assertEqual('/en/about/localeurl/',
                utils.locale_path('/about/localeurl/', 'en'))
        self.assertEqual('/en/about/localeurl/',
                utils.locale_path('/about/localeurl/', 'en-us'))
        self.assertEqual('/nl-nl/about/localeurl/',
                utils.locale_path('/about/localeurl/', 'nl-nl'))
        self.assertEqual('/test/independent/bla/bla',
                utils.locale_path('/test/independent/bla/bla', 'en'))


    def test_locale_url(self):
        previous_script_prefix = urlresolvers.get_script_prefix()
        urlresolvers.set_script_prefix("/blah/")
        self.assertEqual('/blah/en/about/localeurl/',
                utils.locale_url('/about/localeurl/'))
        self.assertEqual('/blah/en/about/localeurl/',
                utils.locale_url('/about/localeurl/', 'de'))
        self.assertEqual('/blah/en/about/localeurl/',
                utils.locale_url('/about/localeurl/', 'en'))
        self.assertEqual('/blah/en/about/localeurl/',
                utils.locale_url('/about/localeurl/', 'en-us'))
        self.assertEqual('/blah/nl-nl/about/localeurl/',
                utils.locale_url('/about/localeurl/', 'nl-nl'))
        self.assertEqual('/blah/test/independent/bla/bla',
                utils.locale_url('/test/independent/bla/bla', 'en'))
        urlresolvers.set_script_prefix(previous_script_prefix)



class MiddlewareTestCase(LocaleurlTestCase):
    def setUp(self):
        super(MiddlewareTestCase, self).setUp()
        self.request_factory = test_utils.RequestFactory()
        self.middleware = middleware.LocaleURLMiddleware()


    def test_with_locale(self):
        r1 = self.request_factory.get('/fr/test/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('fr', r1.LANGUAGE_CODE)
        self.assertEqual('/test/', r1.path_info)


    def test_with_sublocale(self):
        r1 = self.request_factory.get('/nl-nl/test/bla/bla/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('nl-nl', r1.LANGUAGE_CODE)
        self.assertEqual('/test/bla/bla/', r1.path_info)


    def test_case_insensitive_locale(self):
        r1 = self.request_factory.get('/nl-NL/test/bla/bla/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('nl-nl', r1.LANGUAGE_CODE)
        self.assertEqual('/test/bla/bla/', r1.path_info)


    def test_non_lowercase_path(self):
        r1 = self.request_factory.get('/nl-nl/TEST/bla/bla/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('nl-nl', r1.LANGUAGE_CODE)
        self.assertEqual('/TEST/bla/bla/', r1.path_info)


    def test_locale_independent_url(self):
        r1 = self.request_factory.get('/test/independent/bla/bla/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('en-gb', r1.LANGUAGE_CODE)
        self.assertEqual('/test/independent/bla/bla/', r1.path_info)


    def test_locale_specified_on_independent_url_with_query_string(self):
        r1 = self.request_factory.get('/nl-be/test/independent/?foo=bar')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/test/independent/?foo=bar', r2['Location'])


    def test_redirect_with_script_prefix(self):
        previous_script_prefix = urlresolvers.get_script_prefix()
        urlresolvers.set_script_prefix('/prefix/')

        r1 = self.request_factory.get('/nl-be/test/independent/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/prefix/test/independent/', r2['Location'])

        # no infinite redirects
        r3 = self.request_factory.get('/test/independent/')
        r4 = self.middleware.process_request(r3)
        self.assertEqual(None, r4)

        urlresolvers.set_script_prefix(previous_script_prefix)


    def test_redirect_statuscode_302(self):
        self.settings_manager.set(LOCALE_REDIRECT_PERMANENT=False)
        reload(localeurl_settings)

        r1 = self.request_factory.get('/nl-be/test/independent/?foo=bar')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(302, r2.status_code)


    def test_redirect_statuscode_default(self):
        r1 = self.request_factory.get('/nl-be/test/independent/?foo=bar')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)


    def test_check_accept_lang(self):
        self.settings_manager.set(LOCALEURL_USE_ACCEPT_LANGUAGE=True)
        reload(localeurl_settings)

        r1 = self.request_factory.get('/test/', HTTP_ACCEPT_LANGUAGE='fr, de;q=0.8')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/fr/test/', r2['Location'])


    def test_check_session_enabled(self):
        self.settings_manager.set(LOCALEURL_USE_SESSION=True)
        reload(localeurl_settings)

        r1 = self.request_factory.get('/test/')
        r1.session = {'django_language': 'fr'}
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/fr/test/', r2['Location'])



class DefaultPrefixMiddlewareTestCase(MiddlewareTestCase):
    def setUp(self):
        super(DefaultPrefixMiddlewareTestCase, self).setUp()


    def test_no_locale(self):
        r1 = self.request_factory.get('/test/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/en/test/', r2['Location'])


    def test_non_ascii(self):
        r1 = self.request_factory.get('/plaît/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/en/pla%C3%AEt/', r2['Location'])


    def test_with_query_string(self):
        r1 = self.request_factory.get('/test/?somevar=someval')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/en/test/?somevar=someval', r2['Location'])

    def test_with_unescaped_query_string(self):
        # This contains an ISO-88591-2 latin small letter C with cedilla,
        # received in a request declared as windows-1251:
        r1 = self.request_factory.get('/test/?somevar=Mudan\xe7as_recentes')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/en/test/?somevar=Mudan%E7as_recentes', r2['Location'])


    def test_check_session_disabled(self):
        self.settings_manager.set(LOCALEURL_USE_SESSION=False)
        reload(localeurl_settings)

        r1 = self.request_factory.get('/test/')
        r1.session = {'locale': 'fr'}
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/en/test/', r2['Location'])





class NoDefaultPrefixMiddlewareTestCase(MiddlewareTestCase):
    def setUp(self):
        super(NoDefaultPrefixMiddlewareTestCase, self).setUp()
        self.settings_manager.set(PREFIX_DEFAULT_LOCALE=False)
        reload(localeurl_settings)


    def test_default_locale(self):
        r1 = self.request_factory.get('/test/foo/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('en-gb', r1.LANGUAGE_CODE)
        self.assertEqual('/test/foo/', r1.path_info)


    def test_default_locale_specified(self):
        r1 = self.request_factory.get('/en/test/foo/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(301, r2.status_code)
        self.assertEqual('/test/foo/', r2['Location'])


    def test_alternate_locale(self):
        r1 = self.request_factory.get('/fr/test/foo/')
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)
        self.assertEqual('fr', r1.LANGUAGE_CODE)
        self.assertEqual('/test/foo/', r1.path_info)

    def test_check_session_disabled(self):
        self.settings_manager.set(LOCALEURL_USE_SESSION=False)
        reload(localeurl_settings)

        r1 = self.request_factory.get('/test/')
        r1.session = {'locale': 'fr'}
        r2 = self.middleware.process_request(r1)
        self.assertEqual(None, r2)



class TagsTestCase(LocaleurlTestCase):
    def render_template(self, text):
        t = test_utils.TestTemplate(text, libraries=[localeurl_tags.register])
        c = template.Context()
        return t.render(c)


    def test_locale_url_tag(self):
        self.assertRaises(ValueError, self.render_template,
                '{% locale_url "nl" "dummy0" %}')

        self.assertEqual('/en/dummy/', self.render_template(
                '{% locale_url "en-us" "dummy0" %}'))

        self.assertEqual('/fr/dummy/4', self.render_template(
            '{% locale_url "fr" "dummy1" test=4 %}'))

        self.assertEqual('/en/dummy/4', self.render_template(
            '{% locale_url "en" "dummy1" test=4 as testvar %}{{ testvar }}'))


    def test_chlocale_filter(self):
        self.assertEqual('/fr/dummy/', self.render_template(
                '{{ "/dummy/"|chlocale:"fr" }}'))

        self.assertEqual('/en/dummy/', self.render_template(
                '{{"/fr/dummy/"|chlocale:"en-gb"}}'))


    def test_rmlocale_filter(self):
        self.assertEqual('/dummy/', self.render_template(
                '{{ "/dummy/"|rmlocale }}'))

        self.assertEqual('/dummy/', self.render_template(
                '{{ "/nl-be/dummy/"|rmlocale }}'))

        self.assertEqual('/dummy/', self.render_template(
                '{{ "/en/dummy/"|rmlocale }}'))


    def test_locale_url_tag_no_default_prefix(self):
        self.settings_manager.set(PREFIX_DEFAULT_LOCALE=False)
        reload(localeurl_settings)

        self.assertEqual('/dummy/', self.render_template(
                '{% locale_url "en-us" "dummy0" %}'))

        self.assertEqual('/fr/dummy/', self.render_template(
            '{% locale_url "fr" "dummy0" %}'))


    def test_chlocale_filter_no_default_prefix(self):
        self.settings_manager.set(PREFIX_DEFAULT_LOCALE=False)
        reload(localeurl_settings)

        self.assertEqual('/dummy/', self.render_template(
                '{{ "/nl-nl/dummy/"|chlocale:"en-gb" }}'))

        self.assertEqual('/fr/dummy/', self.render_template(
                '{{"/nl-nl/dummy/"|chlocale:"fr"}}'))



class DummyModel(object):
    def __init__(self, num):
        self.num = num


    def get_absolute_url(self):
        return '/dummy/%s/' % self.num



class DummySitemap(LocaleurlSitemap):
    def items(self):
        return [DummyModel(i) for i in range(3)]



class SitemapTestCase(LocaleurlTestCase):
    def setUp(self):
        super(SitemapTestCase, self).setUp()
        class DummySite(object):
            domain = 'www.example.com'
        from django.contrib.sites.models import Site
        self._orig_get_current = Site.objects.get_current
        Site.objects.get_current = lambda: DummySite()


    def test_localeurl_sitemap(self):
        sitemap = DummySitemap('fr')
        self.assertEqual(sitemap.get_urls()[0]['location'],
                         'http://www.example.com/fr/dummy/0/')


    def tearDown(self):
        super(SitemapTestCase, self).tearDown()
        from django.contrib.sites.models import Site
        Site.objects.get_current = self._orig_get_current



class ReverseTestCase(LocaleurlTestCase):
    def test_args_kwargs_none(self):
        url = reverse("dummy0", args=None, kwargs=None)
        self.assertEqual(url, "/en/dummy/")


    def test_args_none(self):
        url = reverse("dummy1", args=None, kwargs=dict(test="test"))
        self.assertEqual(url, "/en/dummy/test")


    def test_kwargs_none(self):
        url = reverse("dummy1", args=["test"], kwargs=None)
        self.assertEqual(url, "/en/dummy/test")


class ViewsTestCase(LocaleurlTestCase):

    urls = 'localeurl.urls'

    def setUp(self):
        super(LocaleurlTestCase, self).setUp()
        self.settings_manager = test_utils.TestSettingsManager()

    def test_change_locale_check_session_enabled(self):
        self.settings_manager.set(LOCALEURL_USE_SESSION=True)
        reload(localeurl_settings)
        self.client.post('/change/', data={'locale': 'de', 'next': '/foo'})
        self.assertEqual("de", self.client.session['django_language'])

    def test_change_locale_check_session_disabled(self):
        self.settings_manager.set(LOCALEURL_USE_SESSION=False)
        reload(localeurl_settings)
        self.client.post('/change/', data={'locale': 'de', 'next': '/foo'})
        self.assertNotEqual("de", self.client.session.get('django_language'))

    def test_change_without_next(self):
        response = self.client.post('/change/', data={'locale': 'de'})
        self.assertEqual(response.status_code, 302)



import subprocess
import os.path

try:
    # don't get confused if our sdist is unzipped in a subdir of some
    # other hg repo
    if os.path.isdir('.hg'):
        p = subprocess.Popen(['hg', 'parents', r'--template={rev}\n'],
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if not p.returncode:
            fh = open('HGREV', 'w')
            fh.write((p.communicate()[0].splitlines()[0] or '').decode('utf-8'))
            fh.close()
except (OSError, IndexError):
    pass

try:
    hgrev = open('HGREV').read()
except IOError:
    hgrev = ''

name = 'localeurl'
authors = 'Joost Cassee, Artiom Diomin and Carl Meyer'
copyright_years = '2008-2010'
version = '2.0.2'
release = version
